#!/usr/bin/python
from ClusterEntity import *
from clusterpkg.cluster_functions import cluPing, countMembers

### 9/8/2003 1. changed loglevel default from 4(Warning)
###             to 5(Notice)
MY_INFO = {
   "WHOAMI"  : "cluquorumd",

   "OPTIONS" : {
      "loglevel"      : { "datatype" : "int",
                          "order"    : 0,
                          "default"  : 5 },
      "pinginterval"  : { "datatype" : "str",
                          "order"    : 1,
                          "default"  : "2" },
      "tiebreaker_ip" : { "datatype" : "str",
                          "order"    : 2 },
   },

   "ADD_GETOPTS"     : [],

   "UPDATE_GETOPTS"  : [ "cluquorumd",
                            "loglevel=",
                            "pinginterval=",
                            "tiebreaker_ip=" ],

   "DELETE_GETOPTS"  : [],
   "LISTALL_GETOPTS" : [],
}

class cluquorumd(ClusterEntity):
   def __init__(self, node):
      ClusterEntity.__init__(self, MY_INFO, node)

   ####################################################################
   # NOTE: In 'validate', WARNING errors must come *Last*, after all
   #       FATAL errors.  This is because the command line tool will
   #       override WARNING's in favor of doing the add or update,
   #       whereas the GUI will present the WARNING in a popup dialog.
   #       Since WARNINGs and ERROR's are 'raise'd, editing stops at
   #       that point.  So for the command line, we want to be sure
   #       that all FATAL checks have already been done, so that if a
   #       WARNING is raised, it can be safely overridden sinced we
   #       know that only WARNING checks remain.
   ####################################################################
   def validate(self, options):
      #print "----> cluquroumd validate NOW!<-----"
      #print "      self.action = %s" % self.action
      #print "      options     = %s" % options

      #######################################################################
      ########################   1. FATAL ERRORS   ##########################
      #######################################################################

      ### 'genericOptionChecks' will raise an error if either the data
      ### doesn't match the datatype(str or int), or the option is
      ### not in the list of valid options for this class.
      ClusterEntity.genericOptionChecks(self, options)

      two_node_cluster_and_no_tiebreaker_ip = FALSE

      if options.has_key("tiebreaker_ip") and options["tiebreaker_ip"] != "":
         tiebreaker_ip = options["tiebreaker_ip"]
         #print "cluquorumd tiebreaker_ip = [%s]" % tiebreaker_ip
         if not cluPing(tiebreaker_ip):
            raise OptionError("FATAL",
                              _("Tiebreaker IP Address not found on network"),
                              "tiebreaker_ip",
                              tiebreaker_ip)

      else:
         ### Tiebreaker IP Address *NOT* submitted ###
         count_members = countMembers(self.node)

         if count_members == 2:
            ### set warning flag, and raise WARNING below ###
            two_node_cluster_and_no_tiebreaker_ip = TRUE

         elif count_members == 4:
            raise OptionError("FATAL",
                              _("Tiebreaker IP Address is required for 4-node clusters"),
                              "tiebreaker_ip",
                              None)

      if options.has_key("loglevel") and options["loglevel"] != "":
         loglevel = options["loglevel"]
         error = FALSE
         try:
            int_loglevel = int(loglevel)
         except:
            error = TRUE
         if not error and \
            (int_loglevel < 0 or int_loglevel > 7):
               error = TRUE
         if error:
            raise OptionError("FATAL",
                              _("""Log Level must be a number from 0 to 7:
  0 - EMERG
  1 - ALERT
  2 - CRIT
  3 - ERR
  4 - WARNING (DEFAULT)
  5 - NOTICE
  6 - INFO
  7 - DEBUG"""),
                              "loglevel",
                              loglevel)

      if options.has_key("pinginterval") and options["pinginterval"] != "":
         pinginterval = options["pinginterval"]
         error = FALSE
         try:
            int_pinginterval = int(pinginterval)
         except:
            error = TRUE

         if not error and \
            (int_pinginterval < 1 or int_pinginterval > 10):
               error = TRUE

         if error:
            raise OptionError("FATAL",
                              _("Ping Interval must be a number from 1 to 10"),
                              "tiebreaker_ip",
                              None)

      #######################################################################
      ##########################   2. WARNINGS   ############################
      #######################################################################
      if two_node_cluster_and_no_tiebreaker_ip:
         raise OptionError("WARNING",
                           _("Tiebreaker IP Address is recommended for 2-node clusters"),
                           "tiebreaker_ip",
                           None)
